@@ -2,7 +2,7 @@
  *
  * Copyright (c) 1999-2008 Daniel Burrows
  * Copyright (c) 2004 Michael Vogt <mvo@debian.org>
- *               2009 Daniel Nicoletti <dantti12@gmail.com>
+ *               2009-2016 Daniel Nicoletti <dantti12@gmail.com>
  *               2012-2015 Matthias Klumpp <matthias@tenstral.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,7 +65,7 @@ AptIntf::AptIntf(PkBackendJob *job) :
     m_restartStat.st_mtime = 0;
 }
 
-bool AptIntf::init()
+bool AptIntf::init(gchar **localDebs)
 {
     const gchar *locale;
     const gchar *http_proxy;
@@ -129,6 +129,11 @@ bool AptIntf::init()
 
     // Create the AptCacheFile class to search for packages
     m_cache = new AptCacheFile(m_job);
+    if (localDebs) {
+        for (int i = 0; i < g_strv_length(localDebs); ++i) {
+            markFileForInstall(localDebs[i]);
+        }
+    }
 
     int timeout = 10;
     // TODO test this
@@ -340,9 +345,9 @@ PkgList AptIntf::filterPackages(const PkgList &packages, PkBitfield filters)
         PkgList ret;
         ret.reserve(packages.size());
 
-        for (PkgList::const_iterator i = packages.begin(); i != packages.end(); ++i) {
-            if (matchPackage(*i, filters)) {
-                ret.push_back(*i);
+        for (const pkgCache::VerIterator &ver : packages) {
+            if (matchPackage(ver, filters)) {
+                ret.push_back(ver);
             }
         }
 
@@ -353,12 +358,12 @@ PkgList AptIntf::filterPackages(const PkgList &packages, PkBitfield filters)
             pkgProblemResolver Fix(*m_cache);
             {
                 pkgDepCache::ActionGroup group(*m_cache);
-                for (PkgList::const_iterator it = ret.begin(); it != ret.end(); ++it) {
+                for (const pkgCache::VerIterator &ver : ret) {
                     if (m_cancel) {
                         break;
                     }
 
-                    m_cache->tryToInstall(Fix, *it, false);
+                    m_cache->tryToInstall(Fix, ver, false);
                 }
             }
 
@@ -377,19 +382,19 @@ PkgList AptIntf::filterPackages(const PkgList &packages, PkBitfield filters)
                 return downloaded;
             }
 
-            for (PkgList::const_iterator verIt = ret.begin(); verIt != ret.end(); ++verIt) {
+            for (const pkgCache::VerIterator &verIt : ret) {
                 bool found = false;
                 for (pkgAcquire::ItemIterator it = fetcher.ItemsBegin(); it < fetcher.ItemsEnd(); ++it) {
                     pkgAcqArchiveSane *archive = static_cast<pkgAcqArchiveSane*>(*it);
                     const pkgCache::VerIterator ver = archive->version();
-                    if ((*it)->Local && *verIt == ver) {
+                    if ((*it)->Local && verIt == ver) {
                         found = true;
                         break;
                     }
                 }
 
                 if (found) {
-                    downloaded.push_back(*verIt);
+                    downloaded.push_back(verIt);
                 }
             }
 
@@ -443,12 +448,12 @@ void AptIntf::emitPackages(PkgList &output, PkBitfield filters, PkInfoEnum state
     output.removeDuplicates();
 
     output = filterPackages(output, filters);
-    for (PkgList::const_iterator it = output.begin(); it != output.end(); ++it) {
+    for (const pkgCache::VerIterator &verIt : output) {
         if (m_cancel) {
             break;
         }
 
-        emitPackage(*it, state);
+        emitPackage(verIt, state);
     }
 }
 
@@ -460,9 +465,9 @@ void AptIntf::emitRequireRestart(PkgList &output)
     // Remove the duplicated entries
     output.removeDuplicates();
 
-    for (PkgList::const_iterator it = output.begin(); it != output.end(); ++it) {
+    for (const pkgCache::VerIterator &verIt : output) {
         gchar *package_id;
-        package_id = utilBuildPackageId(*it);
+        package_id = utilBuildPackageId(verIt);
         pk_backend_job_require_restart(m_job, PK_RESTART_ENUM_SYSTEM, package_id);
         g_free(package_id);
     }
@@ -478,7 +483,7 @@ void AptIntf::emitUpdates(PkgList &output, PkBitfield filters)
     output.removeDuplicates();
 
     output = filterPackages(output, filters);
-    for (PkgList::const_iterator i = output.begin(); i != output.end(); ++i) {
+    for (const pkgCache::VerIterator &verIt : output) {
         if (m_cancel) {
             break;
         }
@@ -487,7 +492,7 @@ void AptIntf::emitUpdates(PkgList &output, PkBitfield filters)
         state = PK_INFO_ENUM_NORMAL;
 
         // let find what kind of upgrade this is
-        pkgCache::VerFileIterator vf = i->FileList();
+        pkgCache::VerFileIterator vf = verIt.FileList();
         std::string origin  = vf.File().Origin() == NULL ? "" : vf.File().Origin();
         std::string archive = vf.File().Archive() == NULL ? "" : vf.File().Archive();
         std::string label   = vf.File().Label() == NULL ? "" : vf.File().Label();
@@ -506,7 +511,7 @@ void AptIntf::emitUpdates(PkgList &output, PkBitfield filters)
             state = PK_INFO_ENUM_ENHANCEMENT;
         }
 
-        emitPackage(*i, state);
+        emitPackage(verIt, state);
     }
 }
 
@@ -760,12 +765,12 @@ void AptIntf::emitDetails(PkgList &pkgs)
     // Remove the duplicated entries
     pkgs.removeDuplicates();
 
-    for (PkgList::const_iterator i = pkgs.begin(); i != pkgs.end(); ++i) {
+    for (const pkgCache::VerIterator &verIt : pkgs) {
         if (m_cancel) {
             break;
         }
 
-        emitPackageDetail(*i);
+        emitPackageDetail(verIt);
     }
 }
 
@@ -879,12 +884,12 @@ void AptIntf::emitUpdateDetail(const pkgCache::VerIterator &candver)
 
 void AptIntf::emitUpdateDetails(const PkgList &pkgs)
 {
-    for (PkgList::const_iterator it = pkgs.begin(); it != pkgs.end(); ++it) {
+    for (const pkgCache::VerIterator &verIt : pkgs) {
         if (m_cancel) {
             break;
         }
 
-        emitUpdateDetail(*it);
+        emitUpdateDetail(verIt);
     }
 }
 
@@ -936,8 +941,8 @@ void AptIntf::getRequires(PkgList &output,
         if (parentVer.end() == false) {
             PkgList deps;
             getDepends(deps, parentVer, false);
-            for (PkgList::const_iterator it = deps.begin(); it != deps.end(); ++it) {
-                if (*it == ver) {
+            for (const pkgCache::VerIterator &depVer : deps) {
+                if (depVer == ver) {
                     if (recursive) {
                         if (!output.contains(parentPkg)) {
                             output.push_back(parentVer);
@@ -1066,10 +1071,8 @@ PkgList AptIntf::getPackagesFromGroup(gchar **values)
             section = section.substr(found + 1);
 
             // Don't insert virtual packages instead add what it provides
-            for (vector<PkGroupEnum>::const_iterator it = groups.begin();
-                 it != groups.end();
-                 ++it) {
-                if (*it == get_enum_group(section)) {
+            for (PkGroupEnum group : groups) {
+                if (group == get_enum_group(section)) {
                     output.push_back(ver);
                     break;
                 }
@@ -1174,20 +1177,33 @@ PkgList AptIntf::searchPackageFiles(gchar **values)
 {
     PkgList output;
     vector<string> packages;
+    string search;
     regex_t re;
-    gchar *search;
-    gchar *values_str;
 
-    values_str = g_strjoinv("$|^", values);
-    search = g_strdup_printf("^%s$",
-                             values_str);
-    g_free(values_str);
-    if(regcomp(&re, search, REG_NOSUB) != 0) {
+    for (uint i = 0; i < g_strv_length(values); ++i) {
+        gchar *value = values[i];
+        if (strlen(value) < 1) {
+            continue;
+        }
+
+        if (!search.empty()) {
+            search.append("|");
+        }
+
+        if (value[0] == '/') {
+            search.append("^");
+            search.append(value);
+            search.append("$");
+        } else {
+            search.append(value);
+            search.append("$");
+        }
+    }
+
+    if(regcomp(&re, search.c_str(), REG_NOSUB) != 0) {
         g_debug("Regex compilation error");
-        g_free(search);
         return output;
     }
-    g_free(search);
 
     DIR *dp;
     struct dirent *dirp;
@@ -1202,16 +1218,18 @@ PkgList AptIntf::searchPackageFiles(gchar **values)
         if (m_cancel) {
             break;
         }
+
         if (ends_with(dirp->d_name, ".list")) {
-            string f = "/var/lib/dpkg/info/" + string(dirp->d_name);
+            string file(dirp->d_name);
+            string f = "/var/lib/dpkg/info/" + file;
             ifstream in(f.c_str());
             if (!in != 0) {
                 continue;
             }
+
             while (!in.eof()) {
                 getline(in, line);
                 if (regexec(&re, line.c_str(), (size_t)0, NULL, 0) == 0) {
-                    string file(dirp->d_name);
                     packages.push_back(file.erase(file.size() - 5, file.size()));
                     break;
                 }
@@ -1222,17 +1240,32 @@ PkgList AptIntf::searchPackageFiles(gchar **values)
     regfree(&re);
 
     // Resolve the package names now
-    for (vector<string>::const_iterator it = packages.begin();
-         it != packages.end(); ++it) {
+    for (const string &name : packages) {
         if (m_cancel) {
             break;
         }
-        const pkgCache::PkgIterator &pkg = (*m_cache)->FindPkg(*it);
-        if (pkg.end() == true) {
-            continue;
+
+        pkgCache::PkgIterator pkg;
+        if (name.find(':') != std::string::npos) {
+            pkg = (*m_cache)->FindPkg(name);
+            if (pkg.end()) {
+                continue;
+            }
+        } else {
+            pkgCache::GrpIterator grp = (*m_cache)->FindGrp(name);
+            for (pkg = grp.PackageList(); pkg.end() == false; pkg = grp.NextPkg(pkg)) {
+                if (pkg->CurrentState == pkgCache::State::Installed) {
+                    break;
+                }
+            }
+
+            if (pkg->CurrentState != pkgCache::State::Installed) {
+                 continue;
+            }
         }
+
         const pkgCache::VerIterator &ver = m_cache->findVer(pkg);
-        if (ver.end() == true) {
+        if (ver.end()) {
             continue;
         }
         output.push_back(ver);
@@ -1333,12 +1366,11 @@ void AptIntf::providesMimeType(PkgList &output, gchar **values)
     regfree(&re);
 
     // resolve the package names
-    for (vector<string>::const_iterator it = packages.begin();
-         it != packages.end(); ++it) {
+    for (const string &package : packages) {
         if (m_cancel) {
             break;
         }
-        const pkgCache::PkgIterator &pkg = (*m_cache)->FindPkg(*it);
+        const pkgCache::PkgIterator &pkg = (*m_cache)->FindPkg(package);
         if (pkg.end() == true) {
             continue;
         }
@@ -1457,6 +1489,28 @@ void AptIntf::emitPackageFiles(const gchar *pi)
     }
 }
 
+void AptIntf::emitPackageFilesLocal(const gchar *file)
+{
+    DebFile deb(file);
+    if (!deb.isValid()){
+        return;
+    }
+
+    gchar *package_id;
+    package_id = pk_package_id_build(deb.packageName().c_str(),
+                                     deb.version().c_str(),
+                                     deb.architecture().c_str(),
+                                     file);
+
+    GPtrArray *files = g_ptr_array_new_with_free_func(g_free);
+    for (auto file : deb.files()) {
+        g_ptr_array_add(files, g_strdup(file.c_str()));
+    }
+    pk_backend_job_files(m_job, package_id, (gchar **) files->pdata);
+
+    g_ptr_array_unref(files);
+}
+
 /**
   * Check if package is officially supported by the current distribution
   */
@@ -1609,9 +1663,9 @@ PkgList AptIntf::checkChangedPackages(bool emitChanged)
 
 pkgCache::VerIterator AptIntf::findTransactionPackage(const std::string &name)
 {
-    for (PkgList::const_iterator it = m_pkgs.begin(); it != m_pkgs.end(); ++it) {
-        if (it->ParentPkg().Name() == name) {
-            return *it;
+    for (const pkgCache::VerIterator &verIt : m_pkgs) {
+        if (verIt.ParentPkg().Name() == name) {
+            return verIt;
         }
     }
 
@@ -1976,41 +2030,39 @@ PkgList AptIntf::resolvePackageIds(gchar **package_ids, PkBitfield filters)
 
         // Check if it's a valid package id
         if (pk_package_id_check(pi) == false) {
-            // Check if we are on multiarch AND if the package name didn't contains the arch field (GDEBI for instance)
-            if (m_isMultiArch && strstr(pi, ":") == NULL) {
+            string name(pi);
+            // Check if the package name didn't contains the arch field
+            if (name.find(':') == std::string::npos) {
                 // OK FindPkg is not suitable on muitarch without ":arch"
                 // it can only return one package in this case we need to
                 // search the whole package cache and match the package
                 // name manually
-                for (pkgCache::PkgIterator pkg = m_cache->GetPkgCache()->PkgBegin(); !pkg.end(); ++pkg) {
+                pkgCache::PkgIterator pkg;
+                pkgCache::GrpIterator grp = (*m_cache)->FindGrp(name);
+                for (pkg = grp.PackageList(); pkg.end() == false; pkg = grp.NextPkg(pkg)) {
                     if (m_cancel) {
                         break;
                     }
 
-                    // check if this is the package we want
-                    if (strcmp(pkg.Name(), pi) != 0) {
-                        continue;
-                    }
-
-                    // Ignore packages that could not be found or that exist only due to dependencies.
-                    if ((pkg.end() == true || (pkg.VersionList().end() && pkg.ProvidesList().end()))) {
+                    // Ignore packages that exist only due to dependencies.
+                    if (pkg.VersionList().end() && pkg.ProvidesList().end()) {
                         continue;
                     }
 
                     const pkgCache::VerIterator &ver = m_cache->findVer(pkg);
                     // check to see if the provided package isn't virtual too
-                    if (ver.end() == false) {
+                    if (!ver.end()) {
                         ret.push_back(ver);
                     }
 
                     const pkgCache::VerIterator &candidateVer = m_cache->findCandidateVer(pkg);
                     // check to see if the provided package isn't virtual too
-                    if (candidateVer.end() == false) {
+                    if (!candidateVer.end()) {
                         ret.push_back(candidateVer);
                     }
                 }
             } else {
-                const pkgCache::PkgIterator &pkg = (*m_cache)->FindPkg(pi);
+                const pkgCache::PkgIterator &pkg = (*m_cache)->FindPkg(name);
                 // Ignore packages that could not be found or that exist only due to dependencies.
                 if (pkg.end() == true || (pkg.VersionList().end() && pkg.ProvidesList().end())) {
                     continue;
@@ -2069,225 +2121,43 @@ void AptIntf::refreshCache()
 
 void AptIntf::markAutoInstalled(const PkgList &pkgs)
 {
-    for (PkgList::const_iterator it = pkgs.begin(); it != pkgs.end(); ++it) {
+    for (const pkgCache::VerIterator &verIt : pkgs) {
         if (m_cancel) {
             break;
         }
 
         // Mark package as auto-installed
-        (*m_cache)->MarkAuto(it->ParentPkg(), true);
+        (*m_cache)->MarkAuto(verIt.ParentPkg(), true);
     }
 }
 
-bool AptIntf::markFileForInstall(const gchar *file, PkgList &install, PkgList &remove)
+bool AptIntf::markFileForInstall(std::string const &file)
 {
-    // We call gdebi to tell us what do we need to install/remove
-    // in order to be able to install this package
-    gint status;
-    gchar **argv;
-    gchar *std_out;
-    gchar *std_err;
-    GError *gerror = NULL;
-    argv = (gchar **) g_malloc(5 * sizeof(gchar *));
-    argv[0] = g_strdup(GDEBI_BINARY);
-    argv[1] = g_strdup("-q");
-    argv[2] = g_strdup("--apt-line");
-    argv[3] = g_strdup(file);
-    argv[4] = NULL;
-
-    gboolean ret;
-    ret = g_spawn_sync(NULL, // working dir
-                       argv, // argv
-                       NULL, // envp
-                       G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
-                       NULL, // child_setup
-                       NULL, // user_data
-                       &std_out, // standard_output
-                       &std_err, // standard_error
-                       &status,
-                       &gerror);
-    int exit_code = WEXITSTATUS(status);
-    //     cout << "DebStatus " << exit_code << " WEXITSTATUS " << WEXITSTATUS(status) << " ret: "<< ret << endl;
-    if (ret) {
-        cout << "std_out " << strlen(std_out) << std_out << endl;
-        cout << "std_err " << strlen(std_err) << std_err << endl;
-    }
-
-    PkgList pkgs;
-    if (!ret) {
-        pk_backend_job_error_code(m_job, PK_ERROR_ENUM_TRANSACTION_ERROR,
-                                  "Spawn of helper '%s' failed: %s",
-                                  argv[0], gerror->message);
-        g_error_free(gerror);
-        return false;
-    } else if (exit_code == 1) {
-        if (strlen(std_out) == 0) {
-            pk_backend_job_error_code(m_job, PK_ERROR_ENUM_TRANSACTION_ERROR, "Error: %s", std_err);
-        } else {
-            pk_backend_job_error_code(m_job, PK_ERROR_ENUM_TRANSACTION_ERROR, "Error: %s", std_out);
-        }
-        return false;
-    } else {
-        // GDebi outputs two lines
-        gchar **lines = g_strsplit(std_out, "\n", 3);
-
-        // The first line contains the packages to install
-        gchar **installPkgs = g_strsplit(lines[0], " ", 0);
-
-        // The second line contains the packages to remove with '-' appended to
-        // the end of the package name
-        gchar **removePkgs = NULL;
-        if (strlen(lines[1]) > 0) {
-            gchar *removeStr = g_strndup(lines[1], strlen(lines[1]) - 1);
-            removePkgs = g_strsplit(removeStr, "- ", 0);
-            g_free(removeStr);
-        }
-
-        // Resolve the packages to install
-        PkBitfield intallFilters;
-        intallFilters = pk_bitfield_from_enums (
-                    PK_FILTER_ENUM_NOT_INSTALLED,
-                    -1);
-        install = resolvePackageIds(installPkgs, intallFilters);
-
-        // Resolve the packages to remove
-        PkBitfield removeFilters;
-        removeFilters = pk_bitfield_from_enums (
-                    PK_FILTER_ENUM_INSTALLED,
-                    -1);
-        remove = resolvePackageIds(removePkgs, removeFilters);
-
-        g_strfreev(lines);
-        g_strfreev(installPkgs);
-        g_strfreev(removePkgs);
-    }
-
-    return true;
+    return m_cache->GetSourceList()->AddVolatileFile(file);
 }
 
-bool AptIntf::installFile(const gchar *path, bool simulate)
+PkgList AptIntf::resolveLocalFiles(gchar **localDebs)
 {
-    if (path == NULL) {
-        g_error ("installFile() path was NULL!");
-        return false;
-    }
-
-    DebFile deb(path);
-    if (!deb.isValid()) {
-        pk_backend_job_error_code(m_job, PK_ERROR_ENUM_TRANSACTION_ERROR, "DEB package is invalid!");
-        return false;
-    }
-
-    if (simulate) {
-        // TODO: Emit signal for to-be-installed package
-        //emit_package("",  PK_FILTER_ENUM_NONE, PK_INFO_ENUM_INSTALLING);
-        return true;
-    }
-
-    string arch = deb.architecture();
-    string aptArch = _config->Find("APT::Architecture");
-
-    // If we are not on multi-arch make sure we got the correct arch package
-    if (!m_isMultiArch && arch != "all" && arch != aptArch) {
-        pk_backend_job_error_code(m_job,
-                                  PK_ERROR_ENUM_INCOMPATIBLE_ARCHITECTURE,
-                                  "Package has wrong architecture, it is %s, but we need %s",
-                                  arch.c_str(),
-                                  aptArch.c_str());
-        return false;
-    }
-
-    // Close the package cache to release the lock
-    m_cache->Close();
-
-    // Build package-id for the new package
-    gchar *deb_package_id = pk_package_id_build(deb.packageName ().c_str (),
-                                                deb.version ().c_str (),
-                                                deb.architecture ().c_str (),
-                                                "local");
-    const gchar *deb_summary = deb.summary ().c_str ();
-
-    gint status;
-    gchar **argv;
-    gchar **envp;
-    gchar *std_out;
-    gchar *std_err;
-    GError *error = NULL;
-
-    argv = (gchar **) g_malloc(4 * sizeof(gchar *));
-    argv[0] = g_strdup("/usr/bin/dpkg");
-    argv[1] = g_strdup("-i");
-    argv[2] = g_strdup(path); //g_strdup_printf("\'%s\'", path);
-    argv[3] = NULL;
-
-    envp = (gchar **) g_malloc(4 * sizeof(gchar *));
-    envp[0] = g_strdup("PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin");
-
-    const gchar *socket = pk_backend_job_get_frontend_socket(m_job);
-    if ((m_interactive) && (socket != NULL)) {
-        envp[1] = g_strdup("DEBIAN_FRONTEND=passthrough");
-        envp[2] = g_strdup_printf("DEBCONF_PIPE=%s", socket);
-        envp[3] = NULL;
-    } else {
-        // we don't have a socket set or are non-interactive, let's fallback to noninteractive
-        envp[1] = g_strdup("DEBIAN_FRONTEND=noninteractive");
-        envp[2] = NULL;
-        envp[3] = NULL;
-    }
-
-    // We're installing the package now...
-    pk_backend_job_package (m_job, PK_INFO_ENUM_INSTALLING, deb_package_id, deb_summary);
-
-    g_spawn_sync(NULL, // working dir
-                 argv,
-                 envp,
-                 G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
-                 NULL, // child_setup
-                 NULL, // user_data
-                 &std_out, // standard_output
-                 &std_err, // standard_error
-                 &status,
-                 &error);
-    int exit_code = WEXITSTATUS(status);
-
-    g_strfreev(envp);
-
-    cout << "DpkgOut: " << std_out << endl;
-    cout << "DpkgErr: " << std_err << endl;
-
-    if (error != NULL) {
-        // We couldn't run dpkg for some reason...
-        pk_backend_job_error_code(m_job,
-                                  PK_ERROR_ENUM_TRANSACTION_ERROR,
-                                  "Failed to run DPKG: %s",
-                                  error->message);
-        return false;
-    }
-
-    // If installation has failed...
-    if (exit_code != 0) {
-        if ((std_out == NULL) || (strlen(std_out) == 0)) {
-            pk_backend_job_error_code(m_job,
-                                      PK_ERROR_ENUM_TRANSACTION_ERROR,
-                                      "Failed: %s",
-                                      std_err);
-        } else {
-            pk_backend_job_error_code(m_job,
-                                      PK_ERROR_ENUM_TRANSACTION_ERROR,
-                                      "Failed: %s",
-                                      std_out);
+    PkgList ret;
+    for (int i = 0; i < g_strv_length(localDebs); ++i) {
+        pkgCache::PkgIterator const P = (*m_cache)->FindPkg(localDebs[i]);
+        if (P.end()) {
+            continue;
         }
-        return false;
+
+        // Set any version providing the .deb as the candidate.
+        for (auto Prv = P.ProvidesList(); Prv.end() == false; Prv++) {
+            ret.push_back(Prv.OwnerVer());
+        }
+
+        // TODO do we need this?
+        // via cacheset to have our usual virtual handling
+        //APT::VersionContainerInterface::FromPackage(&(verset[MOD_INSTALL]), Cache, P, APT::CacheSetHelper::CANDIDATE, helper);
     }
-
-    // Emit data of the now-installed DEB package
-    pk_backend_job_package (m_job, PK_INFO_ENUM_INSTALLED, deb_package_id, deb_summary);
-    g_free (deb_package_id);
-
-    return true;
+    return ret;
 }
 
-bool AptIntf::runTransaction(const PkgList &install, const PkgList &remove, bool markAuto, bool fixBroken, PkBitfield flags, bool autoremove)
+bool AptIntf::runTransaction(const PkgList &install, const PkgList &remove, bool fixBroken, PkBitfield flags, bool autoremove)
 {
     //cout << "runTransaction" << simulate << remove << endl;
 
@@ -2307,27 +2177,22 @@ bool AptIntf::runTransaction(const PkgList &install, const PkgList &remove, bool
     // new scope for the ActionGroup
     {
         pkgDepCache::ActionGroup group(*m_cache);
-        for (PkgList::const_iterator it = install.begin(); it != install.end(); ++it) {
+        for (const pkgCache::VerIterator &verIt : install) {
             if (m_cancel) {
                 break;
             }
 
-            if (!m_cache->tryToInstall(Fix, *it, BrokenFix)) {
+            if (!m_cache->tryToInstall(Fix, verIt, BrokenFix)) {
                 return false;
             }
         }
 
-        for (PkgList::const_iterator it = remove.begin(); it != remove.end(); ++it) {
+        for (const pkgCache::VerIterator &verIt : remove) {
             if (m_cancel) {
                 break;
             }
 
-            m_cache->tryToRemove(Fix, *it);
-        }
-
-        // Mark package dependencies of a local file as auto-installed
-        if (!simulate && markAuto) {
-            markAutoInstalled(install);
+            m_cache->tryToRemove(Fix, verIt);
         }
 
         // Call the scored problem resolver
@@ -2582,7 +2447,7 @@ bool AptIntf::installPackages(PkBitfield flags, bool autoremove)
         _exit(res);
     }
 
-    cout << "PARENT proccess running..." << endl;
+    cout << "PARENT process running..." << endl;
     // make it nonblocking, verry important otherwise
     // when the child finish we stay stuck.
     fcntl(readFromChildFD[0], F_SETFL, O_NONBLOCK);
