@@ -1,7 +1,7 @@
 /* pk-backend-aptcc.cpp
  *
  * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2009-2012 Daniel Nicoletti <dantti12@gmail.com>
+ * Copyright (C) 2009-2016 Daniel Nicoletti <dantti12@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -162,7 +162,9 @@ PkBitfield pk_backend_get_filters(PkBackend *backend)
  */
 gchar** pk_backend_get_mime_types(PkBackend *backend)
 {
-    const gchar *mime_types[] = { "application/x-deb", NULL };
+    const gchar *mime_types[] = { "application/vnd.debian.binary-package",
+                                  "application/x-deb",
+                                  NULL };
     return g_strdupv ((gchar **) mime_types);
 }
 
@@ -344,28 +346,32 @@ void pk_backend_get_files(PkBackend *backend, PkBackendJob *job, gchar **package
 
 static void backend_get_details_thread(PkBackendJob *job, GVariant *params, gpointer user_data)
 {
-    gchar **package_ids;
+    gchar **package_ids = nullptr;
+    gchar **files = nullptr;
     PkRoleEnum role;
     role = pk_backend_job_get_role(job);
 
-    g_variant_get(params, "(^a&s)",
-                  &package_ids);
+    if (role == PK_ROLE_ENUM_GET_DETAILS_LOCAL) {
+        g_variant_get(params, "(^a&s)",
+                      &files);
+    } else {
+        g_variant_get(params, "(^a&s)",
+                      &package_ids);
+    }
 
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
-    if (!apt->init()) {
+    if (!apt->init(files)) {
         g_debug ("Failed to create apt cache");
         return;
     }
 
-    if (package_ids == NULL) {
-        pk_backend_job_error_code(job,
-                                  PK_ERROR_ENUM_PACKAGE_ID_INVALID,
-                                  "Invalid package id");
-        return;
-    }
-
     pk_backend_job_set_status(job, PK_STATUS_ENUM_QUERY);
-    PkgList pkgs = apt->resolvePackageIds(package_ids);
+    PkgList pkgs;
+    if (role == PK_ROLE_ENUM_GET_DETAILS_LOCAL) {
+        pkgs = apt->resolveLocalFiles(files);
+    } else {
+        pkgs = apt->resolvePackageIds(package_ids);
+    }
 
     if (role == PK_ROLE_ENUM_GET_UPDATE_DETAIL) {
         apt->emitUpdateDetails(pkgs);
@@ -390,6 +396,29 @@ void pk_backend_get_details(PkBackend *backend, PkBackendJob *job, gchar **packa
     pk_backend_job_thread_create(job, backend_get_details_thread, NULL, NULL);
 }
 
+void pk_backend_get_details_local(PkBackend *backend, PkBackendJob *job, gchar **files)
+{
+    pk_backend_job_thread_create(job, backend_get_details_thread, NULL, NULL);
+}
+
+static void backend_get_files_local_thread(PkBackendJob *job, GVariant *params, gpointer user_data)
+{
+    gchar **files = nullptr;
+    g_variant_get(params, "(^a&s)",
+                  &files);
+
+    AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
+
+    for (int i = 0; i < g_strv_length(files); ++i) {
+        apt->emitPackageFilesLocal(files[i]);
+    }
+}
+
+void pk_backend_get_files_local(PkBackend *backend, PkBackendJob *job, gchar **files)
+{
+    pk_backend_job_thread_create(job, backend_get_files_local_thread, NULL, NULL);
+}
+
 static void backend_get_updates_thread(PkBackendJob *job, GVariant *params, gpointer user_data)
 {
     PkBitfield filters;
@@ -822,11 +851,10 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
         fixBroken = true;
     }
 
-    g_debug("FILE INSTALL: %i", fileInstall);
     pk_backend_job_set_allow_cancel(job, true);
 
     AptIntf *apt = static_cast<AptIntf*>(pk_backend_job_get_user_data(job));
-    if (!apt->init()) {
+    if (!apt->init(full_paths)) {
         g_debug("Failed to create apt cache");
         return;
     }
@@ -834,31 +862,22 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
     pk_backend_job_set_status(job, PK_STATUS_ENUM_QUERY);
     PkgList installPkgs, removePkgs;
 
-    if (fileInstall) {
-        // File installation EXPERIMENTAL
-
-        // GDebi can not install more than one package at time
-        if (g_strv_length(full_paths) > 1) {
-            pk_backend_job_error_code(job,
-                                      PK_ERROR_ENUM_NOT_SUPPORTED,
-                                      "The backend can only proccess one file at time.");
-            return;
-        }
-
-        // get the list of packages to install
-        if (!apt->markFileForInstall(full_paths[0], installPkgs, removePkgs)) {
-            return;
-        }
-
-        cout << "installPkgs.size: " << installPkgs.size() << endl;
-        cout << "removePkgs.size: " << removePkgs.size() << endl;
-
-    } else if (!fixBroken) {
+    if (!fixBroken) {
         // Resolve the given packages
         if (role == PK_ROLE_ENUM_REMOVE_PACKAGES) {
             removePkgs = apt->resolvePackageIds(package_ids);
-        } else {
+        } else if (role == PK_ROLE_ENUM_INSTALL_PACKAGES ||
+                   role == PK_ROLE_ENUM_UPDATE_PACKAGES) {
+            // Updates are like installs for the purposes of resolution, we
+            // want to install the updates essentially.
             installPkgs = apt->resolvePackageIds(package_ids);
+        } else if (role == PK_ROLE_ENUM_INSTALL_FILES) {
+            installPkgs = apt->resolveLocalFiles(full_paths);
+        } else {
+            pk_backend_job_error_code(job,
+                                      PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
+                                      "Could not figure out what to do to apply the change.");
+            return;
         }
 
         if (removePkgs.size() == 0 && installPkgs.size() == 0) {
@@ -873,8 +892,6 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
     bool ret;
     ret = apt->runTransaction(installPkgs,
                               removePkgs,
-                              fileInstall, // Mark newly installed packages as auto-installed
-                              // (they're dependencies of the new local package)
                               fixBroken,
                               transaction_flags,
                               autoremove);
@@ -883,22 +900,6 @@ static void backend_manage_packages_thread(PkBackendJob *job, GVariant *params,
         g_debug("AptIntf::runTransaction() failed: %i", _error->PendingError());
         return;
     }
-
-    if (fileInstall) {
-        // Now perform the installation!
-        gchar *path;
-        for (uint i = 0; i < g_strv_length(full_paths); ++i) {
-            if (apt->cancelled()) {
-                break;
-            }
-
-            path = full_paths[i];
-            if (!apt->installFile(path, simulate)) {
-                cout << "Installation of DEB file " << path << " failed." << endl;
-                return;
-            }
-        }
-    }
 }
 
 /**
@@ -986,55 +987,55 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
                        &enabled);
     }
 
-    SourcesList _lst;
-    if (_lst.ReadSources() == false) {
+    SourcesList sourcesList;
+    if (sourcesList.ReadSources() == false) {
         _error->
                 Warning("Ignoring invalid record(s) in sources.list file!");
         //return false;
     }
 
-    if (_lst.ReadVendors() == false) {
+    if (sourcesList.ReadVendors() == false) {
         _error->Error("Cannot read vendors.list file");
         show_errors(job, PK_ERROR_ENUM_FAILED_CONFIG_PARSING);
         return;
     }
 
-    for (SourcesListIter it = _lst.SourceRecords.begin();
-         it != _lst.SourceRecords.end(); ++it) {
-        if ((*it)->Type & SourcesList::Comment) {
+    for (SourcesList::SourceRecord *souceRecord : sourcesList.SourceRecords) {
+
+        if (souceRecord->Type & SourcesList::Comment) {
             continue;
         }
 
-        string sections = (*it)->joinedSections();
+        string sections = souceRecord->joinedSections();
         
-        string repoId = (*it)->repoId();
+        string repoId = souceRecord->repoId();
 
         if (role == PK_ROLE_ENUM_GET_REPO_LIST) {
             if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_DEVELOPMENT) &&
-                    ((*it)->Type & SourcesList::DebSrc ||
-                     (*it)->Type & SourcesList::RpmSrc ||
-                     (*it)->Type & SourcesList::RpmSrcDir ||
-                     (*it)->Type & SourcesList::RepomdSrc)) {
+                    (souceRecord->Type & SourcesList::DebSrc ||
+                     souceRecord->Type & SourcesList::RpmSrc ||
+                     souceRecord->Type & SourcesList::RpmSrcDir ||
+                     souceRecord->Type & SourcesList::RepomdSrc)) {
                 continue;
             }
 
             pk_backend_job_repo_detail(job,
                                        repoId.c_str(),
-                                       (*it)->niceName().c_str(),
-                                       !((*it)->Type & SourcesList::Disabled));
+                                       souceRecord->niceName().c_str(),
+                                       !(souceRecord->Type & SourcesList::Disabled));
         } else if (repoId.compare(repo_id) == 0) {
             // Found the repo to enable/disable
             found = true;
 
             if (role == PK_ROLE_ENUM_REPO_ENABLE) {
                 if (enabled) {
-                    (*it)->Type = (*it)->Type & ~SourcesList::Disabled;
+                    souceRecord->Type = souceRecord->Type & ~SourcesList::Disabled;
                 } else {
-                    (*it)->Type |= SourcesList::Disabled;
+                    souceRecord->Type |= SourcesList::Disabled;
                 }
 
                 // Commit changes
-                if (!_lst.UpdateSources()) {
+                if (!sourcesList.UpdateSources()) {
                     _error->Error("Could not update sources file");
                     show_errors(job, PK_ERROR_ENUM_CANNOT_WRITE_REPO_CONFIG);
                 }
@@ -1046,14 +1047,13 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
                         return;
                     }
 
-                    PkgList removePkgs = apt->getPackagesFromRepo(*it);
+                    PkgList removePkgs = apt->getPackagesFromRepo(souceRecord);
                     if (removePkgs.size() > 0) {
                         // Install/Update/Remove packages, or just simulate
                         bool ret;
                         ret = apt->runTransaction(PkgList(),
                                                   removePkgs,
                                                   false,
-                                                  false,
                                                   transaction_flags,
                                                   false);
                         if (!ret) {
@@ -1066,10 +1066,10 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
 
                 // Now if we are not simulating remove the repository
                 if (!pk_bitfield_contain(transaction_flags, PK_TRANSACTION_FLAG_ENUM_SIMULATE)) {
-                    _lst.RemoveSource(*it);
+                    sourcesList.RemoveSource(souceRecord);
 
                     // Commit changes
-                    if (!_lst.UpdateSources()) {
+                    if (!sourcesList.UpdateSources()) {
                         _error->Error("Could not update sources file");
                         show_errors(job, PK_ERROR_ENUM_CANNOT_WRITE_REPO_CONFIG);
                     }
@@ -1083,7 +1083,7 @@ static void backend_repo_manager_thread(PkBackendJob *job, GVariant *params, gpo
 
     if ((role == PK_ROLE_ENUM_REPO_ENABLE || role == PK_ROLE_ENUM_REPO_REMOVE) &&
             !found) {
-        _error->Error("Could not found the repository");
+        _error->Error("Could not find the repository");
         show_errors(job, PK_ERROR_ENUM_REPO_NOT_AVAILABLE);
     }
 }
@@ -1167,7 +1167,9 @@ PkBitfield pk_backend_get_roles(PkBackend *backend)
                 PK_ROLE_ENUM_CANCEL,
                 PK_ROLE_ENUM_DEPENDS_ON,
                 PK_ROLE_ENUM_GET_DETAILS,
+                PK_ROLE_ENUM_GET_DETAILS_LOCAL,
                 PK_ROLE_ENUM_GET_FILES,
+                PK_ROLE_ENUM_GET_FILES_LOCAL,
                 PK_ROLE_ENUM_REQUIRED_BY,
                 PK_ROLE_ENUM_GET_PACKAGES,
                 PK_ROLE_ENUM_WHAT_PROVIDES,
@@ -1188,6 +1190,7 @@ PkBitfield pk_backend_get_roles(PkBackend *backend)
                 PK_ROLE_ENUM_REPO_ENABLE,
                 PK_ROLE_ENUM_REPAIR_SYSTEM,
                 PK_ROLE_ENUM_REPO_REMOVE,
+                PK_ROLE_ENUM_INSTALL_FILES,
                 -1);
 
     // only add GetDistroUpgrades if the binary is present
@@ -1195,10 +1198,5 @@ PkBitfield pk_backend_get_roles(PkBackend *backend)
         pk_bitfield_add(roles, PK_ROLE_ENUM_GET_DISTRO_UPGRADES);
     }
 
-    // only add GetDistroUpgrades if the binary is present
-    if (g_file_test(GDEBI_BINARY, G_FILE_TEST_EXISTS)) {
-        pk_bitfield_add(roles, PK_ROLE_ENUM_INSTALL_FILES);
-    }
-
     return roles;
 }
